#!/bin/bash
# This is part of LTSP over WLAN -script for Ubuntu 8.10
# This script should generate wlanmodules.conf -file for you.
# Please run wlmodsconfgen on your wlan enabled laptop.
# And copy the wlanmodules.conf to your server.

# Copyright (C) January 2009 Jussi Keranen

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

kernelname=`uname -r`

  function clean()
  {
    echo -e "Cleaning..."
    if [ -f modules.temp.1 ]; then rm modules.temp.1; fi
    if [ -f modules.temp.2 ]; then rm modules.temp.2; fi
    if [ -f modules.temp.3 ]; then rm modules.temp.3; fi
    if [ -f firmwares.temp ]; then rm firmwares.temp; fi
    if [ -f firmwares.temp.2 ]; then rm firmwares.temp.2; fi
  }

  function get_modules()
  {
  echo -e "Getting modules..."
  for i in `lsmod | sed 's/\([a-zA-Z0-9_-]*\).*/\1/' | sed '1d'`; do echo $i >> modules.temp.1; done

  cat modules.temp.1 | sed 's/_/*/g' | sed 's/-/*/g' > modules.temp.1

  for i in `cat modules.temp.1`; 
    do 
        for j in `find /lib/modules/$kernelname -iname "$i.ko"`;
          do
            if [ ! `echo $j | grep "/initrd"` ] \
            && [ ! `echo $j | grep "/kernel/arch/.*/kernel"` ] \
            && [ ! `echo $j | grep "/kernel/arch/.*/kvm"` ] \
            && [ ! `echo $j | grep "/kernel/arch/.*/oprofile"` ] \
            && [ ! `echo $j | grep "/kernel/drivers/media"` ] \
            && [ ! `echo $j | grep "/kernel/drivers/pci"` ] \
            && [ ! `echo $j | grep "/kernel/drivers/acpi"` ] \
            && [ ! `echo $j | grep "/kernel/drivers/cpufreq"` ] \
            && [ ! `echo $j | grep "/kernel/drivers/hwmon"` ] \
            && [ ! `echo $j | grep "/kernel/drivers/message"` ] \
            && [ ! `echo $j | grep "/kernel/drivers/power"` ] \
            && [ ! `echo $j | grep "/kernel/drivers/video"` ] \
            && [ ! `echo $j | grep "/kernel/drivers/ata"` ] \
            && [ ! `echo $j | grep "/kernel/drivers/i2c"` ] \
            && [ ! `echo $j | grep "/kernel/drivers/mfd"` ] \
            && [ ! `echo $j | grep "/kernel/drivers/rtc"` ] \
            && [ ! `echo $j | grep "/kernel/drivers/virtio"` ] \
            && [ ! `echo $j | grep "/kernel/drivers/atm"` ] \
            && [ ! `echo $j | grep "/kernel/drivers/dca"` ] \
            && [ ! `echo $j | grep "/kernel/drivers/ieee1394"` ] \
            && [ ! `echo $j | grep "/kernel/drivers/scsi"` ] \
            && [ ! `echo $j | grep "/kernel/drivers/w1"` ] \
            && [ ! `echo $j | grep "/kernel/drivers/auxdisplay"` ] \
            && [ ! `echo $j | grep "/kernel/drivers/dma"` ] \
            && [ ! `echo $j | grep "/kernel/drivers/infiniband"` ] \
            && [ ! `echo $j | grep "/kernel/drivers/mmc"` ] \
            && [ ! `echo $j | grep "/kernel/drivers/serial"` ] \
            && [ ! `echo $j | grep "/kernel/drivers/watchdog"` ] \
            && [ ! `echo $j | grep "/kernel/drivers/block"` ] \
            && [ ! `echo $j | grep "/kernel/drivers/edac"` ] \
            && [ ! `echo $j | grep "/kernel/drivers/input"` ] \
            && [ ! `echo $j | grep "/kernel/drivers/mtd"` ] \
            && [ ! `echo $j | grep "/kernel/drivers/spi"` ] \
            && [ ! `echo $j | grep "/kernel/drivers/bluetooth"` ] \
            && [ ! `echo $j | grep "/kernel/drivers/isdn"` ] \
            && [ ! `echo $j | grep "/kernel/drivers/net/3.*"` ] \
            && [ ! `echo $j | grep "/kernel/drivers/net/8.*"` ] \
            && [ ! `echo $j | grep "/kernel/drivers/net/a.*"` ] \
            && [ ! `echo $j | grep "/kernel/drivers/net/b.*"` ] \
            && [ ! `echo $j | grep "/kernel/drivers/net/c.*"` ] \
            && [ ! `echo $j | grep "/kernel/drivers/net/d.*"` ] \
            && [ ! `echo $j | grep "/kernel/drivers/net/e.*"` ] \
            && [ ! `echo $j | grep "/kernel/drivers/net/f.*"` ] \
            && [ ! `echo $j | grep "/kernel/drivers/net/h.*"` ] \
            && [ ! `echo $j | grep "/kernel/drivers/net/i.*"` ] \
            && [ ! `echo $j | grep "/kernel/drivers/net/l.*"` ] \
            && [ ! `echo $j | grep "/kernel/drivers/net/m.*"` ] \
            && [ ! `echo $j | grep "/kernel/drivers/net/n.*"` ] \
            && [ ! `echo $j | grep "/kernel/drivers/net/pcn.*"` ] \
            && [ ! `echo $j | grep "/kernel/drivers/net/ph.*"` ] \
            && [ ! `echo $j | grep "/kernel/drivers/net/pl.*"` ] \
            && [ ! `echo $j | grep "/kernel/drivers/net/pp.*"` ] \
            && [ ! `echo $j | grep "/kernel/drivers/net/q.*"` ] \
            && [ ! `echo $j | grep "/kernel/drivers/net/r.*"` ] \
            && [ ! `echo $j | grep "/kernel/drivers/net/s.*"` ] \
            && [ ! `echo $j | grep "/kernel/drivers/net/t.*"` ] \
            && [ ! `echo $j | grep "/kernel/drivers/net/v.*"` ] \
            && [ ! `echo $j | grep "/kernel/drivers/net/wan"` ] \
            && [ ! `echo $j | grep "/kernel/drivers/net/wd.*"` ] \
            && [ ! `echo $j | grep "/kernel/drivers/net/y.*"` ] \
            && [ ! `echo $j | grep "/kernel/drivers/net/z.*"` ] \
            && [ ! `echo $j | grep "/kernel/drivers/cdrom"` ] \
            && [ ! `echo $j | grep "/kernel/drivers/gpio"` ] \
            && [ ! `echo $j | grep "/kernel/drivers/parport"` ] \
            && [ ! `echo $j | grep "/kernel/drivers/telephony"` ] \
            && [ ! `echo $j | grep "/kernel/drivers/gpu"` ] \
            && [ ! `echo $j | grep "/kernel/drivers/md"` ] \
            && [ ! `echo $j | grep "/kernel/drivers/uio"` ] \
            && [ ! `echo $j | grep "/kernel/drivers/clocksource"` ] \
            && [ ! `echo $j | grep "/kernel/drivers/hid"` ] \
            && [ ! `echo $j | grep "/kernel/drivers/char"` ] \
            && [ ! `echo $j | grep "/kernel/fs"` ] \
            && [ ! `echo $j | grep "/kernel/lib"` ] \
            && [ ! `echo $j | grep "/kernel/net/802"` ] \
            && [ ! `echo $j | grep "/kernel/net/ax25"` ] \
            && [ ! `echo $j | grep "/kernel/net/decnet"` ] \
            && [ ! `echo $j | grep "/kernel/net/ipx"` ] \
            && [ ! `echo $j | grep "/kernel/net/rose"` ] \
            && [ ! `echo $j | grep "/kernel/net/tipc"` ] \
            && [ ! `echo $j | grep "/kernel/net/8021q"` ] \
            && [ ! `echo $j | grep "/kernel/net/bluetooth"` ] \
            && [ ! `echo $j | grep "/kernel/net/econet"` ] \
            && [ ! `echo $j | grep "/kernel/net/irda"` ] \
            && [ ! `echo $j | grep "/kernel/net/netfilter"` ] \
            && [ ! `echo $j | grep "/kernel/net/rxrpc"` ] \
            && [ ! `echo $j | grep "/kernel/net/wanrouter"` ] \
            && [ ! `echo $j | grep "/kernel/net/9p"` ] \
            && [ ! `echo $j | grep "/kernel/net/bridge"` ] \
            && [ ! `echo $j | grep "/kernel/net/key"` ] \
            && [ ! `echo $j | grep "/kernel/net/netrom"` ] \
            && [ ! `echo $j | grep "/kernel/net/sched"` ] \
            && [ ! `echo $j | grep "/kernel/net/appletalk"` ] \
            && [ ! `echo $j | grep "/kernel/net/core"` ] \
            && [ ! `echo $j | grep "/kernel/net/ipv4"` ] \
            && [ ! `echo $j | grep "/kernel/net/lapb"` ] \
            && [ ! `echo $j | grep "/kernel/net/packet"` ] \
            && [ ! `echo $j | grep "/kernel/net/sctp"` ] \
            && [ ! `echo $j | grep "/kernel/net/x25"` ] \
            && [ ! `echo $j | grep "/kernel/net/atm"` ] \
            && [ ! `echo $j | grep "/kernel/net/dccp"` ] \
            && [ ! `echo $j | grep "/kernel/net/ipv6"` ] \
            && [ ! `echo $j | grep "/kernel/net/llc"` ] \
            && [ ! `echo $j | grep "/kernel/net/sunrpc"` ] \
            && [ ! `echo $j | grep "/kernel/net/xfrm"` ] \
            && [ ! `echo $j | grep "/kernel/sound"` ] \
            && [ ! `echo $j | grep "/kernel/ubuntu/atl2"` ] \
            && [ ! `echo $j | grep "/kernel/ubuntu/dm-loop"` ] \
            && [ ! `echo $j | grep "/kernel/ubuntu/et131x"` ] \
            && [ ! `echo $j | grep "/kernel/ubuntu/heci"` ] \
            && [ ! `echo $j | grep "/kernel/ubuntu/via_chrome9"` ] \
            && [ ! `echo $j | grep "/kernel/ubuntu/aufs"` ] \
            && [ ! `echo $j | grep "/kernel/ubuntu/dm-raid4-5"` ] \
            && [ ! `echo $j | grep "/kernel/ubuntu/gfs"` ] \
            && [ ! `echo $j | grep "/kernel/ubuntu/iscsitarget"` ] \
            && [ ! `echo $j | grep "/kernel/ubuntu/ndiswrapper"` ] \
            && [ ! `echo $j | grep "/kernel/ubuntu/squashfs"` ] \
            && [ ! `echo $j | grep "/kernel/ubuntu/compcache"` ] \
            && [ ! `echo $j | grep "/kernel/ubuntu/drbd"` ] \
            && [ ! `echo $j | grep "/kernel/ubuntu/gnbd"` ] \
            && [ ! `echo $j | grep "/kernel/ubuntu/lirc"` ] \
            && [ ! `echo $j | grep "/kernel/ubuntu/qc-usb"` ] \
            && [ ! `echo $j | grep "/kernel/ubuntu/unionfs"` ];
                then
		 base=$(basename $j)
		 temp="${base%.*}"
                 echo "${temp:-$base}" >> modules.temp.2
                 echo "    ${temp:-$base}"
                fi
          done
    done

  echo -e "\n## MODULES" > modules.temp.3

  for i in `cat modules.temp.2`;
    do
       echo "manual_add_modules $i" >> modules.temp.3
    done
  }


  function get_firmware()
  {
  for i in `cat modules.temp.2 | sed 's/\([a-zA-Z0-9_-]...\).*/\1/'`;
    do 
      for j in `find /lib/firmware -iname "$i*"`;
      do
          echo "mkdir -p \${DESTDIR}$(dirname $j)" >> firmwares.temp.2
          echo "cp -r $j \${DESTDIR}$(dirname $j)" >> firmwares.temp.2 
      done
    done
  if [ -f firmwares.temp.2 ];
  then 
      awk '!x[$0]++' firmwares.temp.2 > firmwares.temp; 
      echo -e "Getting firmware..."
  fi
  }


  function gen_conf()
  {
  echo -e "Generating wlanmodules.conf..."
  cat > wlanmodules.conf <<- EOF
#!/bin/sh

# We don't have any prerequirements
case \$1 in
prereqs)
        exit 0
        ;;
esac

. /usr/share/initramfs-tools/hook-functions
EOF

  cat modules.temp.3 >> wlanmodules.conf
  if [ -f firmwares.temp ]; 
  then 
    echo -e "\n## FIRMWARES" >> wlanmodules.conf; cat firmwares.temp >> wlanmodules.conf; 
  fi
  chmod +x wlanmodules.conf
  }  


  function if_acer()
  {
  for i in `cat modules.temp.1`;
    do
      if [ `echo $i | grep acer` ]; 
      then
        echo -e "Adding acer_wmi..." 
        echo -e "\n## FOR ACER RADIO TURN ON\nforce_load acer_wmi wireless=1" >> wlanmodules.conf;
      fi
    done
  }


  function if_ndiswrapper()
  {
  for i in `cat modules.temp.1`;
    do
      if [ `echo $i | grep ndiswrapper` ];
      then
        echo -e "Adding ndiswrapper..."
        echo -e "\n## NDISWRAPPER\n\
cp -r /etc/ndiswrapper \${DESTDIR}/etc\n\
mkdir -p \${DESTDIR}/usr/sbin\n\
copy_exec /usr/sbin/ndiswrapper-1.9 /usr/sbin\n\
copy_exec /usr/sbin/ndiswrapper /usr/sbin\n\
copy_exec /sbin/loadndisdriver /sbin\n\
copy_exec /sbin/loadndisdriver-1.9 /sbin\n\
ln -s \${DESTDIR}/usr/sbin/ndiswrapper-1.9 \${DESTDIR}/sbin/ndiswrapper-1.9\n\
ln -s \${DESTDIR}/usr/sbin/ndiswrapper \${DESTDIR}/sbin/ndiswrapper\n\
force_load ndiswrapper" >> wlanmodules.conf;
      fi
    done
  }


## MAIN
echo -e "Welcome to the wlanmodules.conf -generator version 0.8\n(C) January 2009 Jussi Keranen\nPlease make sure you have connected to\
 your network via the wlan card\nwhich you want to boot to LTSP.\n\nPress enter to start..."; read;
clean;
get_modules;
get_firmware;
gen_conf;
if_acer;
if_ndiswrapper;
clean;
echo -e "\
\nDone! Thanks for using wlanmodules.conf -generator.\nPlease copy wlanmodules.conf to your LTSP-server\nand run the LTSP_over_WLAN -script and generate\n\
initrd.img with wlan stuff.\nPlease send comments to jussiker at gmail dot com\n"
